/**
 * This file is a part of the project : CorbaTrace
 *
 * It's under LGPL licence.
 */

package corbaTrace.hello;

import java.util.*;
import java.io.*;
import org.omg.CORBA.*;


import corbaTrace.InterceptorClient;

/**
 * Test file
 * The Client Side
 * @author Etienne Juliot
 * @version 0.1
 */
public class Client { 


    public static void main(String args[]) {
	

	InterceptorClient.init();

	Properties props = System.getProperties(); 
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB"); 
	props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	
	int status = 0;
	org.omg.CORBA.ORB orb = null;
	try {
	    orb = org.omg.CORBA.ORB.init(args, props); 
	    status = run(orb);
	} 
	catch(Exception ex) { 
	    ex.printStackTrace(); 
	    status = 1;
	}

	if(orb != null) {
	    try { 
		orb.destroy();
	    } 
	    catch(Exception ex) { 
		ex.printStackTrace(); 
		status = 1; 
	    }
	} 
	System.exit(status);
    }

    static int run(ORB orb) {
	String message = new String("helloClient");
	//InterceptorClient interceptorClient = new InterceptorClient();

	org.omg.CORBA.Object obj = null; 
	try { 
	    String refFile = "Hello.ref"; 
	    BufferedReader in = new BufferedReader(new FileReader(refFile)); 
	    String ref = in.readLine();
	    obj = orb.string_to_object(ref); 
	} 
	catch(IOException ex) { 
	    ex.printStackTrace(); 
	    return 1; 
	}

	obj = InterceptorClient.set_logging_policy_on(obj, orb);
	InterceptorClient.activate_log(orb, message);

	try {

	    Hello hello = HelloHelper.narrow(obj); 
	    
	    System.out.println("Receive : " + hello.say_hello("Titi"));;
	    System.out.println("Receive2 : " + hello.say_hello("Titi"));;
	    
	    return 0; 
	} catch(Exception e) {
	    System.err.println("Cannot connect to server");
	    e.printStackTrace(); 
	    return -1;
	}
    } 

}
