package corbaTrace.log2xmi;

import java.util.*;
import java.io.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.*;

import corbaTrace.xmi.*;  // for creating XMI structure
import corbaTrace.log2xmi.messages.*;  // for manipulating temporarily incomplete messages

/**
 * It's our handler for use with SAX parsing.
 * It is also responsible for manipulating temporarily incomplete messages read from logs.
 * 
 * @author Nicolas Lemoullec
 * @author Antoine Parra del Pozo
 */
public class LogHandler extends DefaultHandler {
    
    /** the XMI document to use for parsing. */
    private XmiDocument xmiDocument;
    private XmiModel xmiModel;
    
    /** the lists of incomplete messages */
    private MessageListSource sourceMessages;
    private MessageListDestination destinationMessages;

    private boolean isSource;
    private Message currentMessage;
    private String currentElement; // to know in what level of tag we are

    private LinkedList optionsListTmp1, optionsListTmp2;
    private MessageOption optionTmp1, optionTmp2;

    private Hashtable definedObjects;  // to know objects already encountered.
    private Hashtable definedClasses;  // to know classes (and their operations) already encountered.

    // constructor
    public LogHandler(XmiDocument xmiDocument, XmiModel xmiModel) {
	super();
	sourceMessages = new MessageListSource();
	destinationMessages = new MessageListDestination();
	definedObjects = new Hashtable();
	definedClasses = new Hashtable();
	this.xmiDocument = xmiDocument;
	this.xmiModel = xmiModel;
        currentMessage = null;
	System.out.println("\n\n ======================================================="
			   + "\n ================== complete messages ==================");
    }
    
    // change the XmiDocument in use (dangerous !).
    //    public void setXmiDocument(XmiDocument xmiDocument) {
    //        this.xmiDocument = xmiDocument;
    //    }
    
    
    //===========================================================
    // SAX DocumentHandler methods
    //===========================================================

    /** executed at the beginning of the document */
    public void startDocument()
	throws SAXException {
    }
    
    
    /** executed when the log is completely read */
    public void endDocument()
	throws SAXException {
    }
    
    // executed when a open tag has been detected
    public void startElement(String namespaceURI,
                             String lName, // local name
                             String qName, // qualified name
                             Attributes attrs)
	throws SAXException {
        String eName = lName; // element name
        if ("".equals(eName)) eName = qName; // namespaceAware = false
	String aName = null;
	if( eName.equalsIgnoreCase("MESSAGE") ) {
	    currentElement = "MESSAGE";
	    currentMessage = new Message();
	    if (attrs != null) {
		for (int i = 0; i < attrs.getLength(); i++) {
		    aName = attrs.getLocalName(i); // Attr name 
		    if ("".equals(aName)) aName = attrs.getQName(i);
		    if (aName.equalsIgnoreCase("IDMESSAGE"))
			currentMessage.setMessageId(attrs.getValue(i));
		    else if (aName.equalsIgnoreCase("TYPE"))
			currentMessage.setMessageType(attrs.getValue(i));
		}
	    }
	    if ((currentMessage.getMessageType() == -1) || (currentMessage.getMessageId() == null))
		currentMessage = null;  // something was getting wrong (bad attributes)
	} else if (currentMessage != null) {
	    if( eName.equalsIgnoreCase("LOCAL_OBJECT") ) {
		currentElement = "LOCAL_OBJECT";
		for (int i = 0; i < attrs.getLength(); i++) {
		    aName = attrs.getLocalName(i); // Attr name 
		    if ("".equals(aName)) aName = attrs.getQName(i);
		    if (aName.equalsIgnoreCase("IDOBJECT")) {
			if ((currentMessage.getMessageType() == Message.RECEIVE_REQUEST)
			    || (currentMessage.getMessageType() == Message.RECEIVE_REPLY)) {
			    currentMessage.setDestinationObjectId(attrs.getValue(i));   // destination part ot the message (receive)
			    isSource = false;
			} else {
			    currentMessage.setSourceObjectId(attrs.getValue(i));     // source part of the message (send)
			    isSource = true;
			}
		    } else if (aName.equalsIgnoreCase("DATE")) {
			if ((currentMessage.getMessageType() == Message.RECEIVE_REQUEST)
			    || (currentMessage.getMessageType() == Message.RECEIVE_REPLY))
			    currentMessage.setReceivingMessageDate(attrs.getValue(i));
			else
			    currentMessage.setSendingMessageDate(attrs.getValue(i));
		    }
		}
	    } else if( eName.equalsIgnoreCase("DISTANT_OBJECT") ) {
		currentElement = "DISTANT_OBJECT";
		if ((currentMessage.getMessageType() == Message.RECEIVE_REQUEST)
		    || (currentMessage.getMessageType() == Message.RECEIVE_REPLY)) {
		    for (int i = 0; i < attrs.getLength(); i++) {
			aName = attrs.getLocalName(i); // Attr name 
			if ("".equals(aName)) aName = attrs.getQName(i);
			if (aName.equalsIgnoreCase("IDOBJECT")) {
			    currentMessage.setSourceObjectId(attrs.getValue(i));
			} else if (aName.equalsIgnoreCase("DATE")) {
			    currentMessage.setSendingMessageDate(attrs.getValue(i));
			}
		    }
		} else if (currentMessage.getMessageType()==Message.SEND_REPLY) { // we KNOW the destination object (we reply to its request)
		    for (int i = 0; i < attrs.getLength(); i++) {
			aName = attrs.getLocalName(i); // Attr name 
			if ("".equals(aName)) aName = attrs.getQName(i);
			if (aName.equalsIgnoreCase("IDOBJECT")) {
			    currentMessage.setDestinationObjectId(attrs.getValue(i));
			} else if (aName.equalsIgnoreCase("DATE")) {
			    currentMessage.setReceivingMessageDate(attrs.getValue(i));  // !!!!!! approximative. Should be more than that
			}
		    }
		} // else no infos are useful
	    } else if( eName.equalsIgnoreCase("OPTIONS") ) {
		currentElement = "OPTIONS";
		// nothing special to do !  We will add options one by one in the message, that's all.
	    } else if( eName.equalsIgnoreCase("REQUEST_ID") ) {
		currentElement = "REQUEST_ID";
		addOptiontoCurrentMessage("request_id", attrs);
	    } else if( eName.equalsIgnoreCase("OPERATION") ) {
		currentElement = "OPERATION";
		optionTmp1 = addOptiontoCurrentMessage("operation", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // arguments list for that option (its value)
	    } else if( eName.equalsIgnoreCase("ARGUMENT") ) {
		if (currentElement.equals("OPERATION"))
		    currentElement = "ARGUMENT";
		if (currentElement.equals("ARGUMENT"))
		    optionsListTmp1.add(constructOptionMessage("argument", attrs));  // adds a new attribute for "Operation"
		// else nothing
	    } else if( eName.equalsIgnoreCase("EXCEPTIONS") ) {
		currentElement = "EXCEPTIONS";
		optionTmp1 = addOptiontoCurrentMessage("exceptions", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // exceptions list for that option (its value)
	    } else if( eName.equalsIgnoreCase("EXCEPTION") ) {
		if (currentElement.equals("EXCEPTIONS"))
		    currentElement = "EXCEPTION";
		if (currentElement.equals("EXCEPTION"))
		    optionsListTmp1.add(constructOptionMessage("exception", attrs));
	    } else if( eName.equalsIgnoreCase("CONTEXTS") ) {
		currentElement = "CONTEXTS";
		optionTmp2 = addOptiontoCurrentMessage("contexts", attrs);  // reference to the added MessageOption
		optionsListTmp2 = new LinkedList();  // contexts list for that option (its value)
	    } else if( eName.equalsIgnoreCase("CONTEXT") ) {
		if (currentElement.equals("PROPERTY") || currentElement.equals("CONTEXTS"))
		    currentElement = "CONTEXT";
		if (currentElement.equals("CONTEXT")) {
		    optionTmp1 = new MessageOption("context");
		    optionsListTmp2.add(optionTmp1);  // contexts list for that option (its value)
		    optionsListTmp1 = new LinkedList();  // properties list for that context option (its value)
		}
	    } else if( eName.equalsIgnoreCase("PROPERTY") ) {
		if (currentElement.equals("CONTEXT"))
		    currentElement = "PROPERTY";
		if (currentElement.equals("PROPERTY"))
		    optionsListTmp1.add(constructOptionMessage("property", attrs));
	    } else if( eName.equalsIgnoreCase("OPERATION_CONTEXT") ) {
		currentElement = "OPERATION_CONTEXT";		
		optionTmp1 = addOptiontoCurrentMessage("operation_context", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // context properties list for that option (its value)
	    } else if( eName.equalsIgnoreCase("CONTEXT_PROPERTY") ) {
		if (currentElement.equals("OPERATION_CONTEXT"))
		    currentElement = "CONTEXT_PROPERTY";
		if (currentElement.equals("CONTEXT_PROPERTY"))
		    optionsListTmp1.add(constructOptionMessage("context_property", attrs));
	    } else if( eName.equalsIgnoreCase("RESULT") ) {
		currentElement = "RESULT";
		addOptiontoCurrentMessage("result", attrs);
	    } else if( eName.equalsIgnoreCase("RESPONSE_EXPECTED") ) {
		currentElement = "RESPONSE_EXPECTED";
		addOptiontoCurrentMessage("response_expected", attrs);
	    } else if( eName.equalsIgnoreCase("SYNC_SCOPE") ) {
		currentElement = "SYNC_SCOPE";
		addOptiontoCurrentMessage("sync_scope", attrs);
	    } else if( eName.equalsIgnoreCase("REPLY_STATUS") ) {
		currentElement = "REPLY_STATUS";
		addOptiontoCurrentMessage("reply_status", attrs);
	    } else if( eName.equalsIgnoreCase("FORWARD_REFERENCE") ) {
		currentElement = "FORWARD_REFERENCE";
		addOptiontoCurrentMessage("forward_reference", attrs);
	    } else if( eName.equalsIgnoreCase("SLOTS") ) {
		currentElement = "SLOTS";		
		optionTmp1 = addOptiontoCurrentMessage("slots", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // slots list for that option (its value)
	    } else if( eName.equalsIgnoreCase("SLOT") ) {
		if (currentElement.equals("SLOTS"))
		    currentElement = "SLOT";
		if (currentElement.equals("SLOT"))
		    optionsListTmp1.add(constructOptionMessage("slot", attrs));
	    } else if( eName.equalsIgnoreCase("REQUEST_SERVICE_CONTEXTS") ) {
		currentElement = "REQUEST_SERVICE_CONTEXTS";		
		optionTmp1 = addOptiontoCurrentMessage("request_service_contexts", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // service contexts list for that option (its value)
	    } else if( eName.equalsIgnoreCase("REPLY_SERVICE_CONTEXTS") ) {
		currentElement = "REPLY_SERVICE_CONTEXTS";
		optionTmp1 = addOptiontoCurrentMessage("reply_services_contexts", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // service contexts list for that option (its value)
	    } else if( eName.equalsIgnoreCase("SERVICE_CONTEXT") ) {
		if (currentElement.equals("REQUEST_SERVICE_CONTEXTS") || currentElement.equals("REPLY_SERVICE_CONTEXTS"))
		    currentElement = "SERVICE_CONTEXT";
		if (currentElement.equals("SERVICE_CONTEXT"))
		    optionsListTmp1.add(constructOptionMessage("service_context", attrs));
	    } else if( eName.equalsIgnoreCase("SENDING_EXCEPTION") ) {
		currentElement = "SENDING_EXCEPTION";
		addOptiontoCurrentMessage("sending_exception", attrs);
	    } else if( eName.equalsIgnoreCase("OBJECT_ID") ) {
		currentElement = "OBJECT_ID";
		addOptiontoCurrentMessage("object_id", attrs);
	    } else if( eName.equalsIgnoreCase("ADAPTER_ID") ) {
		currentElement = "ADAPTER_ID";
		addOptiontoCurrentMessage("adapter_id", attrs);
	    } else if( eName.equalsIgnoreCase("TARGET_MOST_DERIVED_INTERFACE") ) {
		currentElement = "TARGET_MOST_DERIVED_INTERFACE";
		addOptiontoCurrentMessage("target_most_derived_interface", attrs);
	    } else if( eName.equalsIgnoreCase("SERVER_POLICIES") ) {
		currentElement = "SERVER_POLICIES";
		optionTmp1 = addOptiontoCurrentMessage("server_policies", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // policies list for that option (its value)
	    } else if( eName.equalsIgnoreCase("TARGET_IS_A") ) {
		currentElement = "TARGET_IS_A";
		addOptiontoCurrentMessage("target_is_a", attrs);
	    } else if( eName.equalsIgnoreCase("TARGET") ) {
		currentElement = "TARGET";
		addOptiontoCurrentMessage("target", attrs);
	    } else if( eName.equalsIgnoreCase("EFFECTIVE_TARGET") ) {
		currentElement = "EFFECTIVE_TARGET";
		addOptiontoCurrentMessage("effective_target", attrs);
	    } else if( eName.equalsIgnoreCase("EFFECTIVE_PROFILE") ) {
		currentElement = "EFFECTIVE_PROFILE";
		addOptiontoCurrentMessage("effective_profile", attrs);
	    } else if( eName.equalsIgnoreCase("RECEIVED_EXCEPTION") ) {
		currentElement = "RECEIVED_EXCEPTION";
		addOptiontoCurrentMessage("received_exception", attrs);
	    } else if( eName.equalsIgnoreCase("RECEIVED_EXCEPTION_ID") ) {
		currentElement = "RECEIVED_EXCEPTION_ID";
		addOptiontoCurrentMessage("received_exception_id", attrs);
	    } else if( eName.equalsIgnoreCase("EFFECTIVE_COMPONENTS") ) {
		currentElement = "EFFECTIVE_COMPONENTS";
		optionTmp1 = addOptiontoCurrentMessage("effective_components", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // effective components list for that option (its value)
	    } else if( eName.equalsIgnoreCase("EFFECTIVE_COMPONENT") ) {
		if (currentElement.equals("EFFECTIVE_COMPONENTS"))
		    currentElement = "EFFECTIVE_COMPONENT";
		if (currentElement.equals("EFFECTIVE_COMPONENT"))
		    optionsListTmp1.add(constructOptionMessage("effective_component", attrs));
	    } else if( eName.equalsIgnoreCase("REQUEST_POLICIES") ) {
		currentElement = "REQUEST_POLICIES";
		optionTmp1 = addOptiontoCurrentMessage("request_policies", attrs);  // reference to the added MessageOption
		optionsListTmp1 = new LinkedList();  // policies list for that option (its value)
	    } else if( eName.equalsIgnoreCase("POLICY") ) {
		if ((currentElement.equals("REQUEST_POLICIES")) || (currentElement.equals("SERVER_POLICIES")))
		    currentElement = "POLICY";
		if (currentElement.equals("POLICY"))
		    optionsListTmp1.add(constructOptionMessage("policy", attrs));
	    } // else it's a wrong element name.
	} // else it's a wrong element name.
    }
    
    // ----------------------------------------------------------------------

    /** executed when a close tag has been detected */
    public void endElement(String namespaceURI,
                           String sName, // simple name
                           String qName  // qualified name
			   )
	throws SAXException {
        String eName = sName; // element name
        if ("".equals(sName)) eName = qName; // namespaceAware = false
	if( eName.equalsIgnoreCase("MESSAGE") ) {
            // we check wether that message is the other part of
	    // another one. In that case, we join them and add the completed
	    // message to the XmiDocument, else, we keep it in the right list
	    // of incomplete messages.
	    if (currentMessage != null)
		checkandjoinCurrentMessage();
	    // the message has been treated
	    currentMessage = null;
	} else if( eName.equalsIgnoreCase("OPERATION")) {
	    if (currentElement.equals("ARGUMENT"))
		 updateCurrentOptionList1();  // add a list of its arguments as a new property
	} else if( eName.equalsIgnoreCase("EXCEPTIONS")) {
	    if (currentElement.equals("EXCEPTION"))
		 updateCurrentOptionList1();  // add a list of its esceptions as a new property
	} else if( eName.equalsIgnoreCase("CONTEXTS")) {
	    if (currentElement.equals("CONTEXT") || (currentElement.equals("PROPERTY")))
		 updateCurrentOptionList2();
	} else if( eName.equalsIgnoreCase("CONTEXT")) {
	    if (currentElement.equals("PROPERTY"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("OPERATION_CONTEXT")) {		
	    if (currentElement.equals("CONTEXT_PROPERTY"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("SLOTS")) {		
	    if (currentElement.equals("SLOT"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("REQUEST_SERVICE_CONTEXTS")) {
	    if (currentElement.equals("SERVICE_CONTEXT"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("REPLY_SERVICE_CONTEXTS")) {
	    if (currentElement.equals("SERVICE_CONTEXT"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("SERVER_POLICIES")) {
	    if (currentElement.equals("POLICY"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("EFFECTIVE_COMPONENTS")) {
	    if (currentElement.equals("EFFECTIVE_COMPONENT"))
		 updateCurrentOptionList1();
	} else if( eName.equalsIgnoreCase("REQUEST_POLICIES")) {
	    if (currentElement.equals("POLICY"))
		 updateCurrentOptionList1();
	} // else nothing special to do.
    }
    
    /** executed when a #PCDATA or #CDATA has been detected beetween "start" and "end" tags */
    public void characters(char buf[], int offset, int length)
	throws SAXException {
	/* String s = new String(buf, offset, length);
	   if (!s.trim().equals("")) {
	   if( currentElement.equalsIgnoreCase("") ) {
	   }
	   else if( currentElement.equalsIgnoreCase("") ) {
	   }
	   }
	*/
    }
    
    //===========================================================
    // Utility Methods ...
    //===========================================================

    /**
     * general method for adding attributes as properties of the option (vector of (name + value)),
     * then adding option to the current message.
     *
     * precond : message is not null
     *
     * attention : no checks relative coherence between option name & attributes are made
     *             (we suppose all attributes are coherent with it !)
     *
     * @return a reference to the newly created MessageOption.
     */
    private MessageOption addOptiontoCurrentMessage(String optionName, Attributes attrs) {
	MessageOption optionTmp = constructOptionMessage(optionName, attrs);
	currentMessage.addMessageOption(optionTmp);
	return optionTmp;
    }


    private MessageOption constructOptionMessage(String optionName, Attributes attrs) {
	MessageOption optionTmp = new MessageOption(optionName);
	String aName;
	for (int i = 0; i < attrs.getLength(); i++) {
	    aName = attrs.getLocalName(i); // Attr name 
	    if ("".equals(aName))
		optionTmp.addProperty(attrs.getQName(i), attrs.getValue(i));
	    else
		optionTmp.addProperty(aName, attrs.getValue(i));
	}
	return optionTmp;
    }


    private void updateCurrentOptionList1() {
	optionTmp1.addProperty(optionTmp1.getName(), optionsListTmp1);
	optionsListTmp1 = null;
	optionTmp1 = null;
    }

    private void updateCurrentOptionList2() {
	optionTmp2.addProperty(optionTmp2.getName(), optionsListTmp2);
	optionsListTmp2 = null;
	optionTmp2 = null;
    }


    /** 
     * this methods tries to complete a message with the part of current Message (which is incomplete)
     * and the other if already parsed (and also incomplete).
     *
     * or a destination part of the complete message.
     */
    private void checkandjoinCurrentMessage() {
	Message completeMessage;
	if (isSource) {  // if that message is a source part of the message
	    completeMessage = destinationMessages.joinWithSource(currentMessage);
	} else {  // if that message is a destination part of the message
	    completeMessage = sourceMessages.joinWithDestination(currentMessage);
	}
	if (completeMessage != null) {  // the other part has been joined to it.
	    // add the message to the XmiDocument.
	    addXmiMessage(completeMessage);
	} else {
	    if (isSource)
		sourceMessages.addMessage(currentMessage);
	    else
		destinationMessages.addMessage(currentMessage);
	}
    }


    private void addXmiMessage(Message m) {
// XXXXXX Attention : mthode  amliorer !!!!
// comme on a besoin de mmoriser tout pour pouvoir ajouter operation, ( cause des doublons non gr par les classes de la lib xmi),
// a complique.
// ensuite le modele est cr par logsToXmi (pass en paramtre au constructeur) et un diagramme de squence dans le constructeur.

	System.out.println(m.toString());

	String srcIdObject = m.getSourceObjectId();
	String destIdObject = m.getDestinationObjectId();
	XmiClassifierRole destRole = null, srcRole = null;
	XmiClass destClass = null; // for adding new classes & roles

	// check whether our communicating objects (class instances) are already defined.
	if (srcIdObject != null)
	    srcRole = (XmiClassifierRole)definedObjects.get(srcIdObject);
	if (destIdObject != null) {
	    destRole = (XmiClassifierRole)definedObjects.get(destIdObject);
	    destClass = (XmiClass)definedClasses.get(destIdObject);
	}

//XXXXXX quel nom de classe utiliser ?????

	// adds the new roles & classes if necessary
	if ((srcRole == null) && (srcIdObject != null)) {
	    // object has not been defined in this XMI document : we add a new class & a new instance (role).
	    XmiClass classTmp = new XmiClass(xmiDocument, srcIdObject);
	    xmiModel.addClass(classTmp);
	    definedClasses.put(srcIdObject, classTmp);  // adds the new class
	    srcRole = new XmiClassifierRole(xmiDocument, srcIdObject, classTmp);;
	    xmiModel.getSeqDiag().addClassifierRole(srcRole);  // adds this role to the diagram sequence of this xmiModel
	    definedObjects.put(srcIdObject, srcRole);  // adds the new role
	}
	if ((destRole == null) && (destIdObject != null)) {
	    // object has not been defined in this XMI document : we add a new class & a new instance (role).
	    destClass = new XmiClass(xmiDocument, destIdObject);
	    xmiModel.addClass(destClass);
	    definedClasses.put(destIdObject, destClass);  // adds the new class
	    destRole = new XmiClassifierRole(xmiDocument, destIdObject, destClass);;
	    xmiModel.getSeqDiag().addClassifierRole(destRole);  // adds this role to the diagram sequence of this xmiModel
	    definedObjects.put(destIdObject, destRole); // adds the new role
	}


	// for the destClass, gets all its operation, and checks whether this message operation is defined.
	// (if not, adds it for this class). Idem for Actions.
	if (destClass != null) {
	    // gets operations vector from this class.
	    Vector ops = destClass.getOperations();  // gets operations already defined for this class
	    XmiOperation opTmp = null;
	    // gets the name of the operation for this message
	    String operationOfMessage = m.getOperationName();
	    if (operationOfMessage != null) {
		// we check if it's already defined for this class.
		int cpt = ops.size() - 1;
		boolean exit = false;
		while (!exit && (cpt >= 0)) {
//XXXXXXX pour l'instant aucune verif sur les arguments n'est effectue (pas encore implant dans XmiOperation)
//XXXXXXX seul le nom sert  identifier l'opration
		    opTmp = (XmiOperation)(ops.elementAt(cpt));
		    exit = (operationOfMessage.equals(opTmp.getName()));
		    cpt--;
		}
		if (!exit) {  // we add this new operation
		    opTmp = new XmiOperation(xmiDocument, operationOfMessage);
		    destClass.addOperation(opTmp);
		} // else : operation already defined.

	    }

	    // creates an action for this message & operation
//XXXXXXX j'ai pas compris  quoi a servait les actions ????
//XXXXXXX donc l il cre une action  chaque nouveau message (super optimisation)
	    XmiCallAction actionTmp = new XmiCallAction(xmiDocument, "action", opTmp);
	    xmiModel.addCallAction(actionTmp);

	    // adds the message m to this sequence diagram.
	    xmiModel.getSeqDiag().addMessage(new XmiMessage(xmiDocument,m.getMessageId(),srcRole,destRole,actionTmp));

	}
    }


    /** adds all incomplete message to the XMI document */
    public void addIncompleteMessagestoXMI() {
    }


    /** ask just to print infos about incomplete messages still in memory */
    public void showIncompleteMessages() {
	System.out.println("\n\n ============================================================"
			   + "\n ================ incomplete source messages ================\n"
			   + sourceMessages.toString());
	System.out.println("\n\n ================================================================="
			   + "\n ================ incomplete destination messages ================\n"
			   + destinationMessages.toString());
	System.out.println("\n\n");
    }

}
