package corbaTrace.log2xmi;

import java.util.*;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import corbaTrace.xmi.*;
import corbaTrace.log2xmi.stream.*;

/**
 *
 * @author Nicolas Lemoullec
 * @author Antoine Parra del Pozo
 */
class LogstoXmi {
    
    /** lists of LogStreams objects to parse in XMI  */
    private LinkedList logs;
    
    /** the parser for our logs */
    private LogSaxParser parserLogSax;
    
    /** the XMI document for these logs */
    private XmiDocument xmiDocument;
    private String xmiModelName;
    
    /** the XMI handler which treats and joins incompletes messages during the parsing */
    private LogHandler saxHandlerForXmi;

    // constructors
    public LogstoXmi() {
           logs = new LinkedList();
	   xmiDocument = new XmiDocument();  // create the XMI document that will be generated 
	                                     // after all (ore parts of) logs have been parsed.
	   xmiModelName = "test";
	   XmiModel xmiModel=new XmiModel(xmiDocument, xmiModelName);
	   xmiDocument.addModel(xmiModel);
	   XmiSequenceDiagram xmiSequenceDiag = new XmiSequenceDiagram(xmiDocument, "Sequence Diagram");
	   xmiModel.addSequenceDiagram(xmiSequenceDiag);
	   saxHandlerForXmi = new LogHandler(xmiDocument, xmiModel);
           parserLogSax = new LogSaxParser(saxHandlerForXmi);    // init parser SAX parser (factory, etc.) with this handler to use.
	   //XXXXXXXXXXXXXX        LogSaxParser parserLogSax = new LogSaxParser("com.sun.xml.parser.Parser");
	   // to let the user choose its own Sax implementation (class). Deprecated since SAX2/JDK1.4
    }
    
    public LogstoXmi(String []files) {
	this();
	for (int i=0; i<files.length; ++i) {
	    addLog(files[i]);
	}
    }
    
    /**
     * adds a new log to its list
     *
     */
    public void addLog(String log) {
	// to identify the type of source :  url, file, ...
	LogStream logTmp = null;
	switch (identifyLog(log)) {
	case LogFile.FILE_ID :
	    logTmp = new LogFile(log);
	    break;
	case LogUrl.URL_ID :
	    logTmp = new LogUrl(log);
	    break;
	}
	
//XXXXX peut etre aussi tre utilis par exemple pour detecter si le fichier est un log ou un fichier xmi,
//XXXXX donc pour choisir le bon parseur en fonction (crer un XmiFile et faire des tests de type avec getStreamType() par exemple).
             if (logTmp != null)
		 logs.add(logTmp);
        }
    
    public void parseAllLogs() {
	LogStream currentStream = null;
	try {
	    ListIterator l = logs.listIterator();
             while (l.hasNext()) {
		 currentStream = (LogStream)l.next();
		 java.io.InputStream i = currentStream.getStream();
		 if (i != null)
		     parserLogSax.startParsingFile(i);
             }
	} catch (SAXException e) {
	    System.err.println(">>> error while parsing (org.xml.sax.SAXException) : \n" + e.getMessage());
	} catch (java.io.IOException e) {
	    System.err.println(">>> I/O error : I/O error while trying to read log file " + currentStream.getStringValue());
	}
        }
    
    /**
     * save all parsed logs (until now) to an XMI file.
     *
     */
    public void generateXmiFile(String outputfile, boolean addRoseExtensions) {
	saxHandlerForXmi.showIncompleteMessages();
	// saxHandlerForXmi.addIncompleteMessagestoXMI();

	// generate or not Rose extensions for Xmi
	if (addRoseExtensions)   //add extensions for Rational Rose
	    xmiDocument.createRoseExtensions();

	// save Xmi tree in a file
	System.out.println("file "+outputfile+" saved on disk");
	xmiDocument.writeToFile(outputfile);
    }


    private short identifyLog(String s) {
	// checks if it can be converted to an URL.
	try {
	    java.net.URL x = new java.net.URL(s);
	    // no exception : it can be converted : it's an url
 	    return (LogUrl.URL_ID);
	} catch (java.net.MalformedURLException e) {
	    // it's not an URL : we consider it is a file.
	    return (LogFile.FILE_ID);
	}
    }
}
