package corbaTrace.log2xmi.messages;

import java.util.*;

/**
 *
 * @author Nicolas Lemoullec
 * @author Antoine Parra del Pozo
 */
public class MessageListDestination extends MessageList {

    /** 
     * tries to find the other part of the message (destination part)
     * and if found, delete that part from the list and return a joined Message
     * of the two parts.
     * @param the source of the incomplete message
     * @return null if destination part of this source message is not found
     */
     public Message joinWithSource(Message source) {
	 ListIterator l = messagesList.listIterator();
	 Message msgTmp = null;
	 boolean found = false;
	 String supposedSourceObjectId = source.getSourceObjectId();
	 String supposedSendingMessageDate = source.getSendingMessageDate();
	 String supposedMessageId = source.getMessageId();
	 String idTmp=null, dateTmp=null;

	 if ((supposedSourceObjectId != null) 
	     && (supposedSendingMessageDate != null)
	     && (supposedMessageId != null)) {

	     // 1) tries to find the destination part of that source.
	     while (!found && l.hasNext()) {
		 msgTmp = (Message)l.next();
		 idTmp = msgTmp.getSourceObjectId();	
		 dateTmp = msgTmp.getSendingMessageDate();
		 if ((idTmp != null) && (dateTmp != null))
//		 found = (supposedMessageId.equals(msgTmp.getMessageId())
		 found = (supposedSourceObjectId.equals(idTmp)
			  && supposedSendingMessageDate.equals(dateTmp));
	     }

	     if (found) {
		 // 2) we delete it from the list
		 l.remove();
		 
		 // 3) we complete its informations with destination's
		 Vector sourceOptions = source.getMessageOptionList();
		 MessageOption srcOptionTmp = null;
		 MessageOption optTmp = null;
		 for (int i=0; i<sourceOptions.size(); ++i) {
		     srcOptionTmp = (MessageOption)sourceOptions.elementAt(i);
		     if (! msgTmp.optionIsDefined(srcOptionTmp.getName()))
			 msgTmp.addMessageOption(srcOptionTmp);
		 }
	     } else {
		 return null;// returns null (message stays incomplete)
	     }
	 } // else error : we neither add it to this incomplete list, nor join it with a destination message.
	 // we return null.
	 
         return msgTmp;
     }
}
