package corbaTrace.log2xmi.messages;

import java.util.*;

/**
 *
 * @author Nicolas Lemoullec
 * @author Antoine Parra del Pozo
 */
public class MessageListSource extends MessageList {

    /** 
     * tries to find the other part of the message (destination part)
     * and if found, delete that part from the list and return a joined Message
     * of the two parts.
     * @param the source of the incomplete message
     * @return null if destination part of this source message is not found
     */
     public Message joinWithDestination(Message destination) {
	 ListIterator l = messagesList.listIterator();
	 Message msgTmp = null;
	 boolean found = false;
	 String supposedSourceObjectId = destination.getSourceObjectId();
	 String supposedSendingMessageDate = destination.getSendingMessageDate();
	 String supposedMessageId = destination.getMessageId();

	 if ((supposedSourceObjectId != null) 
	     && (supposedSendingMessageDate != null)
	     && (supposedMessageId != null)) {

	     // 1) tries to find the source part of that destination.
	     while (!found && l.hasNext()) {
		 msgTmp = (Message)l.next();

//		 found = (supposedMessageId.equals(msgTmp.getMessageId())
		 found = (supposedSourceObjectId.equals(msgTmp.getSourceObjectId())
			  && supposedSendingMessageDate.equals(msgTmp.getSendingMessageDate()));
	     }
	     if (found) {
		 // 2) we delete it from the list
		 l.remove();
		 
		 // 3) we complete its informations with destination's
		 msgTmp.setDestinationObjectId(destination.getDestinationObjectId());
		 msgTmp.setReceivingMessageDate(destination.getReceivingMessageDate());
		 
		 Vector destinationOptions = destination.getMessageOptionList();
		 MessageOption destOptionTmp = null;
		 MessageOption optTmp = null;
		 for (int i=0; i<destinationOptions.size(); ++i) {
		     destOptionTmp = (MessageOption)destinationOptions.elementAt(i);
		     if (! msgTmp.optionIsDefined(destOptionTmp.getName()))
			 msgTmp.addMessageOption(destOptionTmp);
		 }
	     } else {
		 return null;// returns null (message stays incomplete)
	     }
	 } // else error : we neither add it to this incomplete list, nor join it with a source message.
	   // we return null.
	 
	 return msgTmp;
     }

}
