package corbaTrace.xmi;

import org.w3c.dom.*;
import com.sun.xml.tree.XmlDocument;
import java.util.Vector;

public class XmiClassifierRole{

  private String gid;
  private String name;
  private Element me;
  private Element receivedParent;
  private Element sentParent;
  private XmiDocument doc;
    private Vector messages; //all messages in chronological order
    private Vector receivedMessages;

  public XmiClassifierRole(XmiDocument doc, String name, XmiClass type){
    this.name=name;
    this.doc=doc;
    this.messages=new Vector();
    this.receivedMessages=new Vector();
    initClassifierRole(name, type);
  }

    public Vector getMessages(){
	return this.messages;
    }

    public Vector getReceivedMessages(){
	return this.receivedMessages;
    }

  public Element getElement(){
    return this.me;
  }

  public String getGid(){
    return this.gid;
  }

  public String getName(){
    return this.name;
  }

  private void initClassifierRole(String name,XmiClass theClass){
    /**
    <Behavioral_Elements.Collaborations.ClassifierRole xmi.id = 'G.5'>
      <Foundation.Core.ModelElement.name>InstanceObjet2</Foundation.Core.ModelElement.name>
      <Foundation.Core.ModelElement.visibility xmi.value = 'private'/>
      <Foundation.Core.GeneralizableElement.isRoot xmi.value = 'false'/>
      <Foundation.Core.GeneralizableElement.isLeaf xmi.value = 'false'/>
      <Foundation.Core.GeneralizableElement.isAbstract xmi.value = 'false'/>
      <Behavioral_Elements.Collaborations.ClassifierRole.multiplicity>1..1</Behavioral_Elements.Collaborations.ClassifierRole.multiplicity>
      <Behavioral_Elements.Collaborations.ClassifierRole.message2>

        //addReceivedMessage ici

      </Behavioral_Elements.Collaborations.ClassifierRole.message2>
      <Behavioral_Elements.Collaborations.ClassifierRole.message>

        //addSentMessage ici

      </Behavioral_Elements.Collaborations.ClassifierRole.message>

      <Behavioral_Elements.Collaborations.ClassifierRole.base>
        <Foundation.Core.Class xmi.idref = 'S.10001'/> <!-- Objet2 -->
      </Behavioral_Elements.Collaborations.ClassifierRole.base>

      <Behavioral_Elements.Collaborations.ClassifierRole.availableFeature>
        <Foundation.Core.Operation xmi.idref = 'S.10002'/> <!-- operation1 -->
      </Behavioral_Elements.Collaborations.ClassifierRole.availableFeature>

      <Foundation.Core.ModelElement.taggedValue>
        <Foundation.Extension_Mechanisms.TaggedValue>
          <Foundation.Extension_Mechanisms.TaggedValue.tag>persistence</Foundation.Extension_Mechanisms.TaggedValue.tag>
          <Foundation.Extension_Mechanisms.TaggedValue.value>transient</Foundation.Extension_Mechanisms.TaggedValue.value>
        </Foundation.Extension_Mechanisms.TaggedValue>
      </Foundation.Core.ModelElement.taggedValue>
    </Behavioral_Elements.Collaborations.ClassifierRole>
    **/

    this.me=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole");
    this.gid=doc.getUniqueGid();
    this.me.setAttribute("xmi.id",this.gid);
    Element elem2=doc.createElement("Foundation.Core.ModelElement.name");
    elem2.appendChild(doc.createTextNode(name));
    this.me.appendChild(elem2);
    elem2=doc.createElement("Foundation.Core.ModelElement.visibility");
    elem2.setAttribute("xmi.value","public");
    this.me.appendChild(elem2);
    elem2=doc.createElement("Foundation.Core.GeneralizableElement.isRoot");
    elem2.setAttribute("xmi.value","true");
    this.me.appendChild(elem2);
    elem2=doc.createElement("Foundation.Core.GeneralizableElement.isLeaf");
    elem2.setAttribute("xmi.value","true");
    this.me.appendChild(elem2);
    elem2=doc.createElement("Foundation.Core.GeneralizableElement.isAbstract");
    elem2.setAttribute("xmi.value","false");
    this.me.appendChild(elem2);
    elem2=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole.multiplicity");
    elem2.appendChild(doc.createTextNode("1..1"));
    this.me.appendChild(elem2);
    this.receivedParent=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole.message2");
    this.me.appendChild(this.receivedParent);
    this.sentParent=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole.message");
    this.me.appendChild(this.sentParent);
    elem2=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole.base");
    this.me.appendChild(elem2);
    Element elem3=doc.createElement("Foundation.Core.Class");
    elem3.setAttribute("xmi.idref",theClass.getSid()); //the id of the class
    elem2.appendChild(elem3);
    elem2=doc.createElement("Behavioral_Elements.Collaborations.ClassifierRole.availableFeature");
    this.me.appendChild(elem2);

    //for each operation of the class
    XmiOperation op;
    for(int i=0;i<theClass.getOperations().size();i++){
      op=(XmiOperation)theClass.getOperations().elementAt(i);
      elem3=doc.createElement("Foundation.Core.Operation");
      elem3.setAttribute("xmi.idref", op.getSid());
      elem2.appendChild(elem3);
    }

    elem2=doc.createElement("Foundation.Core.ModelElement.taggedValue");
    this.me.appendChild(elem2);
    elem3=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue");
    elem2.appendChild(elem3);
    Element elem4=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue.tag");
    elem4.appendChild(doc.createTextNode("persistence"));
    elem3.appendChild(elem4);
    elem4=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue.value");
    elem4.appendChild(doc.createTextNode("transient"));
    elem3.appendChild(elem4);
  }

  public void addSentMessage(XmiMessage message){
    /**
    <Behavioral_Elements.Collaborations.Message xmi.idref = 'G.9'/> <!-- operation1( ) -->
    **/
    Element elem=doc.createElement("Behavioral_Elements.Collaborations.Message");
    elem.setAttribute("xmi.idref",message.getGid());
    this.sentParent.appendChild(elem);
    this.messages.addElement(message);
  }

  public void addReceivedMessage(XmiMessage message){
    /**
    <Behavioral_Elements.Collaborations.Message xmi.idref = 'G.10'/> <!-- operation1( ) -->
    **/
    Element elem=doc.createElement("Behavioral_Elements.Collaborations.Message");
    elem.setAttribute("xmi.idref",message.getGid());
    this.receivedParent.appendChild(elem);
    this.messages.addElement(message);
    this.receivedMessages.addElement(message);
  }

}
