package corbaTrace.xmi;

import org.w3c.dom.*;
import com.sun.xml.tree.XmlDocument;
import java.util.*;

public class XmiSequenceDiagram{

  private Element me;
  private XmiDocument doc;
  private Element roleParent;
  private Element messageParent;
  private Vector roles;
  private String name;
  private String gid;
  private XmiMessage predecessor;
    private Vector messages;

  public XmiSequenceDiagram(XmiDocument doc, String name){
    this.doc=doc;
    this.roles=new Vector();
    this.name=name;
    this.predecessor=null;
    this.messages=new Vector();
    initDiagram(name);
  }

  public Element getElement(){
    return this.me;
  }

  public Vector getRoles(){
    return this.roles;
  }

  public String getName(){
    return this.name;
  }
  
  public String getGid(){
    return this.gid;
  }

    public Vector getMessages(){
	return this.messages;
    }

  private void initDiagram(String name){
    //TAKE CARE: the DTD umd.dtd specifies there must be at least 1 message in the diagram
    //(even if no role???)

    /**
      <Behavioral_Elements.Collaborations.Collaboration xmi.id = 'G.2'>
        <Foundation.Core.ModelElement.name>DiagSeq1</Foundation.Core.ModelElement.name>
        <Foundation.Core.ModelElement.visibility xmi.value = 'private'/>
        <Foundation.Core.ModelElement.taggedValue>
          <Foundation.Extension_Mechanisms.TaggedValue>
            <Foundation.Extension_Mechanisms.TaggedValue.tag>DiagramType</Foundation.Extension_Mechanisms.TaggedValue.tag>
            <Foundation.Extension_Mechanisms.TaggedValue.value>SequenceDiagram</Foundation.Extension_Mechanisms.TaggedValue.value>
          </Foundation.Extension_Mechanisms.TaggedValue>
        </Foundation.Core.ModelElement.taggedValue>
        <Foundation.Core.Namespace.ownedElement>

          //addClassifierRole ici

        </Foundation.Core.Namespace.ownedElement>

        <Behavioral_Elements.Collaborations.Collaboration.interaction>
          <Behavioral_Elements.Collaborations.Interaction xmi.id = 'G.14'>
            <Foundation.Core.ModelElement.name></Foundation.Core.ModelElement.name>
            <Foundation.Core.ModelElement.visibility xmi.value = 'private'/>
            <Behavioral_Elements.Collaborations.Interaction.message>

              //addMessage ici

            </Behavioral_Elements.Collaborations.Interaction.message>
          </Behavioral_Elements.Collaborations.Interaction>
        </Behavioral_Elements.Collaborations.Collaboration.interaction>
      </Behavioral_Elements.Collaborations.Collaboration>
    **/

    this.gid=this.doc.getUniqueGid();
    this.me=doc.createElement("Behavioral_Elements.Collaborations.Collaboration");
    this.me.setAttribute("xmi.id",this.gid);
    Element elem=doc.createElement("Foundation.Core.ModelElement.name");
    elem.appendChild(doc.createTextNode(name));
    this.me.appendChild(elem);
    elem=doc.createElement("Foundation.Core.ModelElement.visibility");
    elem.setAttribute("xmi.value","private");
    this.me.appendChild(elem);
    elem=doc.createElement("Foundation.Core.ModelElement.taggedValue");
    this.me.appendChild(elem);
    Element elem2=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue");
    elem.appendChild(elem2);
    Element elem3=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue.tag");
    elem3.appendChild(doc.createTextNode("DiagramType"));
    elem2.appendChild(elem3);
    elem3=doc.createElement("Foundation.Extension_Mechanisms.TaggedValue.value");
    elem3.appendChild(doc.createTextNode("SequenceDiagram"));
    elem2.appendChild(elem3);

    this.roleParent=doc.createElement("Foundation.Core.Namespace.ownedElement");
    this.me.appendChild(this.roleParent);
    elem=doc.createElement("Behavioral_Elements.Collaborations.Collaboration.interaction");
    this.me.appendChild(elem);
    elem2=doc.createElement("Behavioral_Elements.Collaborations.Interaction");
    elem2.setAttribute("xmi.id",doc.getUniqueGid());
    elem.appendChild(elem2);
    elem3=doc.createElement("Foundation.Core.ModelElement.name");
    elem2.appendChild(elem3);
    elem3=doc.createElement("Foundation.Core.ModelElement.visibility");
    elem3.setAttribute("xmi.value","private");
    elem2.appendChild(elem3);
    this.messageParent=doc.createElement("Behavioral_Elements.Collaborations.Interaction.message");
    elem2.appendChild(this.messageParent);

  }

  public void addClassifierRole(XmiClassifierRole role){
      //add the role to the diagram
    this.roleParent.appendChild(role.getElement());
    this.roles.addElement(role);
  }

  public void addMessage(XmiMessage message){
    //add the message to the diagram
    this.messageParent.appendChild(message.getElement());

    //add the message to the hashset
    this.messages.addElement(message);

    //add the message as sent or received to the roles
    message.getSender().addSentMessage(message);
    message.getReceiver().addReceivedMessage(message);

    //add the successor to the message
    if (this.predecessor!=null)
	this.predecessor.setSuccessor(message);

    //add the predecessor to the message
    if (this.predecessor!=null)
	//there is a predecessor
	message.setPredecessor(this.predecessor);

    this.predecessor=message;
  }

  public void addMagicDrawExtensionReference(String gid){
    /**
      <XMI.extension xmi.extender = 'MagicDraw UML 3.6' xmi.extenderID = 'MagicDraw UML 3.6' >
        <mainDiagram xmi.idref = 'ID00000000000e' />
      </XMI.extension>
    **/
    Element elem=doc.createElement("XMI.extension");
    elem.setAttribute("xmi.extender","MagicDraw UML 3.6");
    elem.setAttribute("xmi.extenderID","MagicDraw UML 3.6");
    Element elem2=doc.createElement("mainDiagram");
    elem2.setAttribute("xmi.idref",gid);
    elem.appendChild(elem2);
    this.me.insertBefore(elem,this.roleParent);
  }

}
